<?php

/**
 * @file
 * Git implementation of review interface.
 *
 * @author Jimmy Berry ("boombatower", http://drupal.org/user/214218)
 */

/**
 * Git implementation of VCS interface.
 */
class pifr_client_vcs_git extends pifr_client_vcs {

  /**
   * Array of patch levels to try when patching.
   */
  protected $patch_levels_to_try = array(1, 0);

  /**
   * The patch level (-p0 or -p1) detected.
   */
  protected $patch_level = 0;


  /**
   * Check out a given repo URL into given directory.
   *
   * @param $directory
   *   Directory to check out into.
   * @param unknown_type $url
   *   Repo URL, like git://git.drupal.org/project/drupal.git
   * @param $branch
   *   Branch to check out, like 7.x-1.0.
   */
  public function checkout($directory, $url, $branch) {
    $url = escapeshellarg($url);
    $branch = escapeshellarg($branch);
    if (!empty($directory)) {
      $directory = escapeshellarg($directory);
    }
    // We can use a reference repository to shortcut the wait for a clone.
    // It can have any or all of the projects that might get checked out.
    // This should be an absolute path to the reference repo or a git URL.
    $reference_repo = variable_get('pifr_reference_repository', '');
    if (!empty($reference_repo)) {
      $cmd = "git clone --reference $reference_repo -b $branch $url $directory";
    }
    else {
      $cmd = "git clone --depth 1 -b $branch $url $directory";
    }
    return pifr_client_review::exec($cmd);
  }

  public function annotate($file) {
    $lines = pifr_client_review::exec_output("git annotate $file");

    $map = array();
    $number = 1;
    foreach ($lines as $line) {
      if (preg_match('/.*?\s+\(\s+(.*?)\s+.*?\)/', $line, $match)) {
        $map[$number++] = $match[1];
      }
    }
    return $map;
  }

  public function commit_id() {
    $lines = pifr_client_review::exec_output('git log -n 1');

    foreach ($lines as $line) {
      if (preg_match('/^commit (.*?)$/m', $line, $match)) {
        return $match[1];
      }
    }
    return FALSE;
  }

  /**
   * Apply a patch.
   *
   * This will try patching with the -p possibilities in
   * $this->patch_levels_to_try.
   *
   * @param $patch
   *   The patch file to apply.
   */
  public function apply($patch) {
    foreach($this->patch_levels_to_try as $patch_level) {
      $result = pifr_client_review::exec("patch -p$patch_level -i $patch");
      if ($result) {
        $this->patch_level = $patch_level;
        return TRUE;
      }
    }
    return FALSE;
  }

  /**
   * Inspect the patch to determine which files should have changed.
   *
   * $this->patch_level has already been initialized by $this->apply(), and
   * is the number that was successfully used in "patch -p<number>".
   * We use this to determine how much to strip out of the +++ line.
   *
   * @param $patch
   *   The patch which was applied. (It has to have already been applied to
   *   initialize $this->patch_level.
   */
  public function changed_files($patch) {
    $contents = file_get_contents($patch);
    // This regex looks through the patch file for lines of the format
    // +++ b/install.php
    // and removes the '+++ b/' from them, returning the filenames that have
    // had content added.
    $patch_level = $this->patch_level;
    $regex = '%^[\+-]{3}\s+([^/]\/){' . $patch_level . '}(.*?)\s%m';
    preg_match_all($regex, $contents, $matches, PREG_SET_ORDER);

    $files = array();
    foreach ($matches as $match) {
      // $match[2] is the file. $match[1] is the prefixes.
      if ($match[2] != '/dev/null') {
        $files[] = $match[2];
      }
    }
    return array_unique($files);
  }

  public function get_display($url, $branch) {
    return $url . ' (' . $branch . ')';
  }
}

